-- PeonQuestComplete addon
--  Provides an audio cue on the following events:
--  - completing a quest
--  - completing one of the objectives of a quest (without fully completing that quest)
-- Based on Yoco's QuestSounds http://www.curse.com/addons/wow/questsounds-yoco
-- $Date: 2012/09/08 15:08 $ $Revision: 1.00 $

-- Sound set definitions
local soundSets = {
  peon = { -- this set acts as default for new installations
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peon/PeonReady1.ogg",
  },  
  peasant ={
	questComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantYes1.ogg",
  },
  mix ={
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
  },
  }

-- The saved variables
PQCSndVars = {} -- will be initialized in PLAYER_LOGIN

-- The sounds to play
local sounds = nil -- will be initialized in PLAYER_LOGIN

local me = CreateFrame("Frame")
local events = {}

me.questIndex = 0
me.questId = 0
me.completeCount = 0

local function countCompleteObjectives(index)
  local n = 0
  for i = 1, GetNumQuestLeaderBoards(index) do
    local _, _, finished = GetQuestLogLeaderBoard(i, index)
    if finished then
      n = n + 1
    end
  end
  return n
end

function me:setQuest(index)
  self.questIndex = index
  if index>0 then
    local _, _, _, _, _, _, _, _, id = GetQuestLogTitle(index)
    self.questId = id
    if id and id>0 then
      self.completeCount = countCompleteObjectives(index)
    end
  end
end

function me:checkQuest()
  if self.questIndex>0 then
    local index = self.questIndex
    self.questIndex = 0
    local title, level, _, _, _, _, complete, daily, id = GetQuestLogTitle(index)
    local link = GetQuestLink(index)
    if id==self.questId then
      if id and id>0 then
        local objectivesComplete = countCompleteObjectives(index)
        if complete then
          print("["..level.."] '"..link.."': complete");
          self:Play(sounds and sounds.questComplete)
        elseif objectivesComplete>self.completeCount then
          print("["..level.."] '"..link.."': objective complete ("..objectivesComplete.."/"..GetNumQuestLeaderBoards(index)..")");
          self:Play(sounds and sounds.objectiveComplete)
        else
        end
      end
    end
  end
end

function me:init()
  self:SetScript("OnEvent", function(frame, event, ...)
    local handler = events[event]
    if handler then
      -- dispatch events that were auto-registered by naming convention
      handler(frame, ...)
    end
  end)
  for k,v in pairs(events) do
    self:RegisterEvent(k)
  end
end

function me:Play(sound)
  -- print("Playing:", sound)
  if sound and sound~="" then
    PlaySoundFile(sound);
  end
end

function me:SetSoundSet(setName, silent)
  if setName and soundSets[setName] then
    PQCSndVars.ActiveSet = setName
    sounds = soundSets[setName]
    if not(silent) then
      print("Switching to sound set |cFF44FF00"..setName.."|r")
      me:Play(sounds.questComplete)
    end
  else
    print("Recognized sound set names are:")
    for k,v in pairs(soundSets) do
      if k==PQCSndVars.ActiveSet then
        print("  "..k.."      (*)")
      else
        print("  "..k)
      end
    end
  end
end

-- ................................................................
-- event handlers

function events:UNIT_QUEST_LOG_CHANGED(unit)
  if unit=="player" then
    me:checkQuest()
  end
end

function events:QUEST_WATCH_UPDATE(index)
  -- This event triggers just *before* the changes are registered
  -- in the quest log, giving a great opportunity to quantify changes
  -- in the subsequent UNIT_QUEST_LOG_CHANGED
  me:setQuest(index)
end

function events:PLAYER_LOGIN()
  -- Saved variables are stable now
  if not(PQCSndVars.ActiveSet) then
    PQCSndVars.ActiveSet = "peon"
  end
  me:SetSoundSet(PQCSndVars.ActiveSet, true)
end

-- ................................................................
-- command interface

SLASH_QUESTSOUNDS1 = "/pqc"
SLASH_QUESTSOUNDS2 = "/peonquestcomplete"

function SlashCmdList.QUESTSOUNDS(msg, editbox)
  local cmd, rest = msg:match("^(%S*)%s*(.-)$")
  me:Command(cmd, rest)
end

function me:Command(command, arg)
  local handler = self["CMD"..command]
  if handler then
    handler(self, arg)
  else
    if command~="" then
      print("Unknown questsounds command '"..command.."'");
    end
    print("  /pqc set <soundsetname>")
    print("     Sets the sound set and plays its quest completion sound")
    print("  /pqc set")
    print("     Lists the known sound set names")
    print("  /pqc test <quest|objective>")
    print("     Plays the current 'quest' or 'objective' completion sound")
    print("  The current sound set is |cFF44FF00"..PQCSndVars.ActiveSet.."|r");
  end
end

function me:CMDtest(arg)
  if arg=="1" or arg=="quest" then
    print("Playing |cFFCCCC44quest complete|r sound of sound set |cFF44FF00"..PQCSndVars.ActiveSet.."|r")
    self:Play(sounds and sounds.questComplete)
  elseif arg=="2" or arg=="objective" then
    print("Playing |cFFCCCC44objective complete|r sound of sound set |cFF44FF00"..PQCSndVars.ActiveSet.."|r")
    self:Play(sounds and sounds.objectiveComplete)
  else
    print("  Usage: /pqc test <sound id>")
    print("    Recognized values for <sound id> are:")
    print("      '1' or 'quest' for the quest complete sound")
    print("      '2' or 'objective' for the objective complete sound")
  end
end

function me:CMDset(arg)
  self:SetSoundSet(arg)
end

-- ................................................................
-- must be last line:
me:init()
